@@ -8,9 +8,9 @@ from contextlib import contextmanager
 
 import six
 
-from python_pachyderm.client.pfs.pfs_pb2 import *
-from python_pachyderm.client.pfs.pfs_pb2_grpc import *
-from python_pachyderm.util import commit_from, get_address
+from python_pachyderm.client.pfs import pfs_pb2 as proto
+from python_pachyderm.client.pfs import pfs_pb2_grpc as grpc
+from python_pachyderm.util import commit_from, get_address, get_metadata
 
 
 BUFFER_SIZE = 3 * 1024 * 1024  # 3MB TODO: Base this on some grpc value
@@ -26,16 +26,18 @@ class ExtractValueIterator(object):
 
 
 class PfsClient(object):
-    def __init__(self, host=None, port=None):
+    def __init__(self, host=None, port=None, auth_token=None):
         """
         Creates a client to connect to Pfs
         :param host: The pachd host. Default is 'localhost', which is used with `pachctl port-forward`
         :param port: The port to connect to. Default is 30650
+        :param auth_token: The authentication token; used if authentication is enabled on the cluster. Default to `None`.
         """
-        # If a host or port is not specified, then try to set using environment variables or use the defaults.
+
         address = get_address(host, port)
-        self.channel = grpc.insecure_channel(address)
-        self.stub = APIStub(self.channel)
+        self.metadata = get_metadata(auth_token)
+        self.channel = grpc.grpc.insecure_channel(address)
+        self.stub = grpc.APIStub(self.channel)
 
     def create_repo(self, repo_name, description=None):
         """
@@ -46,7 +48,8 @@ class PfsClient(object):
         :param repo_name: Name of the repo
         :param description: Repo description
         """
-        self.stub.CreateRepo(CreateRepoRequest(repo=Repo(name=repo_name), description=description))
+        req = proto.CreateRepoRequest(repo=proto.Repo(name=repo_name), description=description)
+        self.stub.CreateRepo(req, metadata=self.metadata)
 
     def inspect_repo(self, repo_name):
         """
@@ -54,7 +57,9 @@ class PfsClient(object):
         :param repo_name: Name of the repo
         :return: A RepoInfo object
         """
-        return self.stub.InspectRepo(InspectRepoRequest(repo=Repo(name=repo_name)))
+        req = proto.InspectRepoRequest(repo=proto.Repo(name=repo_name))
+        res = self.stub.InspectRepo(req, metadata=self.metadata)
+        return res
 
     def list_repo(self):
         """
@@ -64,9 +69,10 @@ class PfsClient(object):
         the specified repos as provenance will be returned.
         :return: A list of RepoInfo objects
         """
-        x = self.stub.ListRepo(ListRepoRequest())
-        if hasattr(x, 'repo_info'):
-            return x.repo_info
+        req = proto.ListRepoRequest()
+        res = self.stub.ListRepo(req, metadata=self.metadata)
+        if hasattr(res, 'repo_info'):
+            return res.repo_info
         return []
 
     def delete_repo(self, repo_name=None, force=False, all=False):
@@ -83,12 +89,14 @@ class PfsClient(object):
         """
         if not all:
             if repo_name:
-                self.stub.DeleteRepo(DeleteRepoRequest(repo=Repo(name=repo_name), force=force))
+                req = proto.DeleteRepoRequest(repo=proto.Repo(name=repo_name), force=force)
+                self.stub.DeleteRepo(req, metadata=self.metadata)
             else:
                 raise ValueError("Either a repo_name or all=True needs to be provided")
         else:
             if not repo_name:
-                self.stub.DeleteRepo(DeleteRepoRequest(force=force, all=all))
+                req = proto.DeleteRepoRequest(force=force, all=all)
+                self.stub.DeleteRepo(req, metadata=self.metadata)
             else:
                 raise ValueError("Cannot specify a repo_name if all=True")
 
@@ -111,9 +119,9 @@ class PfsClient(object):
                     initially appear empty.
         :return: Commit object
         """
-        return self.stub.StartCommit(StartCommitRequest(parent=Commit(repo=Repo(name=repo_name),
-                                                                      id=parent),
-                                                        branch=branch))
+        req = proto.StartCommitRequest(parent=proto.Commit(repo=proto.Repo(name=repo_name), id=parent), branch=branch)
+        res = self.stub.StartCommit(req, metadata=self.metadata)
+        return res
 
     def finish_commit(self, commit):
         """
@@ -122,7 +130,9 @@ class PfsClient(object):
         attempts to write to it with PutFile will error.
         :param commit: A tuple or string representing the commit
         """
-        self.stub.FinishCommit(FinishCommitRequest(commit=commit_from(commit)))
+        req = proto.FinishCommitRequest(commit=commit_from(commit))
+        res = self.stub.FinishCommit(req, metadata=self.metadata)
+        return res
 
     @contextmanager
     def commit(self, repo_name, branch=None, parent=None):
@@ -145,7 +155,8 @@ class PfsClient(object):
         :param commit: A tuple or string representing the commit
         :return: CommitInfo object
         """
-        return self.stub.InspectCommit(InspectCommitRequest(commit=commit_from(commit)))
+        req = proto.InspectCommitRequest(commit=commit_from(commit))
+        return self.stub.InspectCommit(req, metadata=self.metadata)
 
     def provenances_for_repo(self, repo_name):
         provenances = {}
@@ -170,14 +181,14 @@ class PfsClient(object):
                     all commits that match the aforementioned criteria are returned.
         :return: A list of CommitInfo objects
         """
-        req = ListCommitRequest(repo=Repo(name=repo_name), number=number)
+        req = proto.ListCommitRequest(repo=proto.Repo(name=repo_name), number=number)
         if to_commit is not None:
             req.to.CopyFrom(commit_from(to_commit))
         if from_commit is not None:
             getattr(req, 'from').CopyFrom(commit_from(from_commit))
-        x = self.stub.ListCommit(req)
-        if hasattr(x, 'commit_info'):
-            return x.commit_info
+        res = self.stub.ListCommit(req, metadata=self.metadata)
+        if hasattr(res, 'commit_info'):
+            return res.commit_info
         return []
 
     def delete_commit(self, commit):
@@ -186,7 +197,8 @@ class PfsClient(object):
         Note it is currently not implemented.
         :param commit: A tuple or string representing the commit
         """
-        self.stub.DeleteCommit(DeleteCommitRequest(commit=commit_from(commit)))
+        req = proto.DeleteCommitRequest(commit=commit_from(commit))
+        self.stub.DeleteCommit(req, metadata=self.metadata)
 
     def flush_commit(self, commits, repos=tuple()):
         """
@@ -204,8 +216,9 @@ class PfsClient(object):
                     will be considered, otherwise all repos are considered.
         :return: An iterator of CommitInfo objects
         """
-        return self.stub.FlushCommit(FlushCommitRequest(commit=[commit_from(c) for c in commits],
-                                                        to_repo=[Repo(name=r) for r in repos]))
+        req = proto.FlushCommitRequest(commit=[commit_from(c) for c in commits], to_repo=[proto.Repo(name=r) for r in repos])
+        res = self.stub.FlushCommit(req, metadata=self.metadata)
+        return res
 
     def subscribe_commit(self, repo_name, branch, from_commit_id=None):
         """
@@ -218,11 +231,12 @@ class PfsClient(object):
 
         :return: Iterator of Commit objects
         """
-        repo = Repo(name=repo_name)
-        req = SubscribeCommitRequest(repo=repo, branch=branch)
+        repo = proto.Repo(name=repo_name)
+        req = proto.SubscribeCommitRequest(repo=repo, branch=branch)
         if from_commit_id is not None:
-            getattr(req, 'from').CopyFrom(Commit(repo=repo, id=from_commit_id))
-        return self.stub.SubscribeCommit(req)
+            getattr(req, 'from').CopyFrom(proto.Commit(repo=repo, id=from_commit_id))
+        res = self.stub.SubscribeCommit(req, metadata=self.metadata)
+        return res
 
     def list_branch(self, repo_name):
         """
@@ -230,9 +244,10 @@ class PfsClient(object):
         :param repo_name: The name of the repo
         :return: A list of Branch objects
         """
-        x = self.stub.ListBranch(ListBranchRequest(repo=Repo(name=repo_name)))
-        if hasattr(x, 'branch_info'):
-            return x.branch_info
+        req = proto.ListBranchRequest(repo=proto.Repo(name=repo_name))
+        res = self.stub.ListBranch(req, metadata=self.metadata)
+        if hasattr(res, 'branch_info'):
+            return res.branch_info
         return []
 
     def set_branch(self, commit, branch_name):
@@ -241,8 +256,8 @@ class PfsClient(object):
         :param commit: A tuple or string representing the commit
         :param branch_name: The name for the branch to set
         """
-        self.stub.SetBranch(SetBranchRequest(commit=commit_from(commit),
-                                             branch=branch_name))
+        res = proto.SetBranchRequest(commit=commit_from(commit), branch=branch_name)
+        self.stub.SetBranch(res, metadata=self.metadata)
 
     def delete_branch(self, repo_name, branch_name):
         """
@@ -252,10 +267,10 @@ class PfsClient(object):
         :param repo_name: The name of the repo
         :param branch_name: The name of the branch to delete
         """
-        self.stub.DeleteBranch(DeleteBranchRequest(repo=Repo(name=repo_name),
-                                                   branch=branch_name))
+        res = proto.DeleteBranchRequest(repo=Repo(name=repo_name), branch=branch_name)
+        self.stub.DeleteBranch(res, metadata=self.metadata)
 
-    def put_file_bytes(self, commit, path, value, delimiter=NONE,
+    def put_file_bytes(self, commit, path, value, delimiter=proto.NONE,
                        target_file_datums=0, target_file_bytes=0):
         """
         Uploads a binary bytes array as file(s) in a certain path
@@ -272,23 +287,27 @@ class PfsClient(object):
         """
 
         if isinstance(value, collections.Iterable) and not isinstance(value, (six.string_types, six.binary_type)):
-            def wrap(v):
-                for x in v:
-                    yield PutFileRequest(file=File(commit=commit_from(commit), path=path),
-                                         value=x,
-                                         delimiter=delimiter,
-                                         target_file_datums=target_file_datums,
-                                         target_file_bytes=target_file_bytes)
+            def wrap(values):
+                for value in values:
+                    yield proto.PutFileRequest(
+                        file=proto.File(commit=commit_from(commit), path=path),
+                        value=value,
+                        delimiter=delimiter,
+                        target_file_datums=target_file_datums,
+                        target_file_bytes=target_file_bytes
+                    )
         else:
-            def wrap(v):
-                for i in range(0, len(v), BUFFER_SIZE):
-                    yield PutFileRequest(file=File(commit=commit_from(commit), path=path),
-                                         value=v[i:i + BUFFER_SIZE],
-                                         delimiter=delimiter,
-                                         target_file_datums=target_file_datums,
-                                         target_file_bytes=target_file_bytes)
+            def wrap(value):
+                for i in range(0, len(value), BUFFER_SIZE):
+                    yield proto.PutFileRequest(
+                        file=proto.File(commit=commit_from(commit), path=path),
+                        value=value[i:i + BUFFER_SIZE],
+                        delimiter=delimiter,
+                        target_file_datums=target_file_datums,
+                        target_file_bytes=target_file_bytes
+                    )
 
-        self.stub.PutFile(wrap(value))
+        self.stub.PutFile(wrap(value), metadata=self.metadata)
 
     def put_file_url(self, commit, path, url, recursive=False):
         """
@@ -300,9 +319,14 @@ class PfsClient(object):
         :param url: The url to download
         :param recursive: allow for recursive scraping of some types URLs for example on s3:// urls.
         """
-        self.stub.PutFile(iter([PutFileRequest(file=File(commit=commit_from(commit), path=path),
-                                               url=url,
-                                               recursive=recursive)]))
+        req = iter([
+            proto.PutFileRequest(
+                file=proto.File(commit=commit_from(commit), path=path),
+                url=url,
+                recursive=recursive
+            )
+        ])
+        self.stub.PutFile(req, metadata=self.metadata)
 
     def get_file(self, commit, path, offset_bytes=0, size_bytes=0, extract_value=True):
         """
@@ -319,13 +343,15 @@ class PfsClient(object):
                     response iterator will return
         :return: An iterator over the file or an iterator over the protobuf responses
         """
-        r = self.stub.GetFile(GetFileRequest(file=File(commit=commit_from(commit),
-                                                       path=path),
-                                             offset_bytes=offset_bytes,
-                                             size_bytes=size_bytes))
+        req = proto.GetFileRequest(
+            file=proto.File(commit=commit_from(commit), path=path),
+            offset_bytes=offset_bytes,
+            size_bytes=size_bytes
+        )
+        res = self.stub.GetFile(req, metadata=self.metadata)
         if extract_value:
-            return ExtractValueIterator(r)
-        return r
+            return ExtractValueIterator(res)
+        return res
 
     def get_files(self, commit, paths, recursive=False):
         """
@@ -339,12 +365,12 @@ class PfsClient(object):
         filtered_file_infos = []
         for path in paths:
             fi = self.inspect_file(commit, path)
-            if fi.file_type == FILE:
+            if fi.file_type == proto.FILE:
                 filtered_file_infos.append(fi)
             else:
                 filtered_file_infos += self.list_file(commit, path, recursive=recursive)
 
-        filtered_paths = [fi.file.path for fi in filtered_file_infos if fi.file_type == FILE]
+        filtered_paths = [fi.file.path for fi in filtered_file_infos if fi.file_type == proto.FILE]
 
         return {path: b''.join(self.get_file(commit, path)) for path in filtered_paths}
 
@@ -355,8 +381,9 @@ class PfsClient(object):
         :param path: Path to file
         :return: A FileInfo object
         """
-        return self.stub.InspectFile(InspectFileRequest(file=File(commit=commit_from(commit),
-                                                                  path=path)))
+        req = proto.InspectFileRequest(file=proto.File(commit=commit_from(commit), path=path))
+        res = self.stub.InspectFile(req, metadata=self.metadata)
+        return res
 
     def list_file(self, commit, path, recursive=False):
         """
@@ -366,13 +393,16 @@ class PfsClient(object):
         :param recursive: If True, continue listing the files for sub-directories
         :return: A list of FileInfo objects
         """
-        file_infos = self.stub.ListFile(ListFileRequest(file=File(commit=commit_from(commit),
-                                                                  path=path))).file_info
+        req = proto.ListFileRequest(
+            file=proto.File(commit=commit_from(commit), path=path)
+        )
+        res = self.stub.ListFile(req, metadata=self.metadata)
+        file_infos = res.file_info
+
         if recursive:
-            dirs = [f for f in file_infos if f.file_type == DIR]
-            files = [f for f in file_infos if f.file_type == FILE]
-            return sum([self.list_file(commit, d.file.path, recursive) for d in dirs],
-                       files)
+            dirs = [f for f in file_infos if f.file_type == proto.DIR]
+            files = [f for f in file_infos if f.file_type == proto.FILE]
+            return sum([self.list_file(commit, d.file.path, recursive) for d in dirs], files)
 
         return list(file_infos)
 
@@ -383,10 +413,10 @@ class PfsClient(object):
         :param pattern:
         :return: A list of FileInfo objects
         """
-        r = self.stub.GlobFile(GlobFileRequest(commit=commit_from(commit),
-                                               pattern=pattern))
-        if hasattr(r, 'file_info'):
-            return r.file_info
+        req = proto.GlobFileRequest(commit=commit_from(commit), pattern=pattern)
+        res = self.stub.GlobFile(req, metadata=self.metadata)
+        if hasattr(res, 'file_info'):
+            return res.file_info
         return []
 
     def delete_file(self, commit, path):
@@ -400,8 +430,9 @@ class PfsClient(object):
         :param commit: A tuple or string representing the commit
         :param path: The path to the file
         """
-        self.stub.DeleteFile(DeleteFileRequest(file=File(commit=commit_from(commit),
-                                                         path=path)))
+        req = proto.DeleteFileRequest(file=proto.File(commit=commit_from(commit), path=path))
+        self.stub.DeleteFile(req, metadata=self.metadata)
 
     def delete_all(self):
-        self.stub.DeleteAll(google_dot_protobuf_dot_empty__pb2.Empty())
+        req = proto.google_dot_protobuf_dot_empty__pb2.Empty()
+        self.stub.DeleteAll(req, metadata=self.metadata)
