@@ -6,26 +6,19 @@ from python_pachyderm.client.pfs import pfs_pb2 as pfs_proto
 
 
 def get_address(host=None, port=None):
-    if host is None or port is None:
-        env_old_host = os.environ.get("PACHD_SERVICE_HOST")
-
-        if env_old_host is not None:
-            import warnings
-            warnings.warn("The environment variable `PACHD_SERVICE_HOST` is deprecated; either explicitly set the host, or use `PACHD_ADDRESS`", DeprecationWarning)
-            host = env_old_host
-
-    if port is None:
-        env_old_port = os.environ.get("PACHD_SERVICE_PORT_API_GRPC_PORT")
-
-        if env_old_port is not None:
-            import warnings
-            warnings.warn("The environment variable `PACHD_SERVICE_PORT_API_GRPC_PORT` is deprecated; either explicitly set the host, or use `PACHD_ADDRESS`", DeprecationWarning)
-            port = env_old_port
-
     if host is not None and port is not None:
         return "{}:{}".format(host, port)
-    else:
-        return os.environ.get("PACHD_ADDRESS", "localhost:30650")
+    return os.environ.get("PACHD_ADDRESS", "localhost:30650")
+
+def get_metadata(auth_token=None):
+    if auth_token is None:
+        auth_token = os.environ.get("PACH_PYTHON_AUTH_TOKEN")
+
+    metadata = []
+    if auth_token is not None:
+        metadata.append(("authn-token", auth_token))
+
+    return metadata
 
 def commit_from(src, allow_just_repo=False):
     if isinstance(src, pfs_proto.Commit):
